/*	Hardware definitions for graphic LCD displays

		PA.0-7	I/O	LCD D0-7

		PE.0	Out	LCD CE_
		PE.4	Out	LCD WR_
		PE.3	Out	LCD RD_

		PE.1	Out	LCD C/D_
*/

/* LCD control signals */
#define	LCD_CD	1
#define	LCD_CE	0
#define	LCD_WR	4
#define	LCD_RD	3


void MsDelay ( int iDelay )
{
	unsigned long ul0;
	ul0 = MS_TIMER;		// get current timer value
	while ( MS_TIMER < ul0 + (unsigned long) iDelay );
}


/* Low level functions */

#asm
;functions require one parameter:
;@sp+2= data to write
;
LCD_WriteCmd::
	call LCD_Busy			; Wait 'till its ready
   jr Write					; C/D returns set (command)
LCD_WriteACmd::
	call LCD_ABusy			; Wait 'till its ready
   jr Write					; C/D returns set (command)

LCD_WriteData::
	call LCD_Busy			; Wait 'till its ready
   jr WriteData
LCD_WriteAData::
	call LCD_ABusy			; Wait 'till its ready
WriteData:
	ld hl,PEDRShadow		; point to control port (shadow)
	ld de,PEDR				; point to control port
	res LCD_CD,(HL)		; Lower C/D (data)
	ioi ldd					; do it

Write:
	ld a,0x84				; PA0-7 = Outputs, put data in bus
	ioi ld (SPCR),a		; write it (no shadow, no one else uses port A)
	ld hl,(sp+2)			; get data value (LSB)
	ld a,l
	ioi ld (PADR),a		; and write it (no shadow, no one else uses port A)
	ld hl,PEDRShadow		; point to control port (shadow)
	ld de,PEDR				; point to control port
	res LCD_CE,(HL)		; Lower CE
	res LCD_WR,(HL)		; Lower WR
	ioi ldd					; do it
	ld hl,PEDRShadow		; point to control port (shadow)
	ld de,PEDR				; point to control port
	set LCD_WR,(HL)		; Rise WR
	set LCD_CE,(HL)		; Rise CE
	ioi ldd					; do it
	ret

LCD_Busy:
	ld e',3					; STA0, STA1
   jr Busy
LCD_ABusy:
	ld e',8					; STA3
Busy:
	ld a,0x80				; PA0-7 = Inputs, read data from bus
	ioi ld (SPCR),a		; write it (no shadow, no one else uses port A)
	ld hl,PEDRShadow		; point to control port (shadow)
	ld de,PEDR				; point to control port
	set LCD_CD,(HL)		; Rise C/D
	ioi ldd					; do it
l1:
	ld hl,PEDRShadow		; point to control port (shadow)
	ld de,PEDR				; point to control port
	res LCD_CE,(HL)		; Lower CE
	res LCD_RD,(HL)		; Lower RD
	ioi ldd					; do it
	ioi ld a,(PADR)		; Read status
	ld hl,PEDRShadow		; point to control port (shadow)
	ld de,PEDR				; point to control port
	set LCD_RD,(HL)		; Rise RD
	set LCD_CE,(HL)		; Rise CE
	ioi ldd					; do it
   ex de',hl				; HL = DE'
	and l						; Check busy (get rid of non interesting bits)
   cp l 						; do check busy bit(s)
	jr nz,l1					; loop while busy
   ret

#endasm


void LCD_init ()
{
	WrPortI ( PEDR,&PEDRShadow,'\B10011011' ); 	// Make sure Outputs are inactive
	WrPortI ( PEDDR,&PEDDRShadow,'\B10011011' );	// PE0,1,3,4,7 = output
	WrPortI ( PEFR, &PEFRShadow, 0 );		// PE: no I/O strobe
	MsDelay ( 1000 );								// wait for LCD to reset itself
   LCD_WriteData(0x0);
   LCD_WriteData(0x0);
	LCD_WriteCmd(0x40); 							// Set TXT home
   LCD_WriteData(0x1e);							// 30x8=240
   LCD_WriteData(0x0);
	LCD_WriteCmd(0x41); 							// Set TXT cols
   LCD_WriteData(0x0);
   LCD_WriteData(0x10);							// 0x1000
	LCD_WriteCmd(0x42); 							// Set GFX home
   LCD_WriteData(0x1e);							// 30x8=240
   LCD_WriteData(0x0);
	LCD_WriteCmd(0x43); 							// Set GFX cols
	LCD_WriteCmd(0x81); 							// TXT XOR GFX, internal CG
	LCD_WriteCmd(0x9C); 							// TXT on, GFX on, no cursor, no blink
}


/* High level functions */


void LCD_address ( int address )
{
	LCD_WriteData(address&0xFF);				// LO byte
	LCD_WriteData((address>>8)&0xFF);		// HI byte
   LCD_WriteCmd(0x24);							// set address
}


void LCD_fill(int count,unsigned char pattern)
{
int i;
	LCD_WriteCmd(0xb0);						// Auto data write
	for(i=0;i<count;i++)
		LCD_WriteAData(pattern);			// fill with pattern
	LCD_WriteACmd(0xb2);						// reset Auto data write
}

void LCD_fillblack()
{
		LCD_address(0x1000);					// GFX screen
		LCD_fill(30*64,0);					// fill with 0's (black)
}

void LCD_fillwhite()
{
		LCD_address(0x1000);					// GFX screen
		LCD_fill(30*64,0xFF); 				// fill with FF's (white)
}

int LCD_print (char *ptr)
{
char c;
int i;

	i=0;
	LCD_WriteCmd(0xb0);						// Auto data write
 	while (c=*ptr++){
		LCD_WriteAData (c-0x20);			// convert ASCII to controller code
      i++;
   }
	LCD_WriteACmd(0xb2);						// reset Auto data write
   return(i);									// return character count
}

void LCD_printat (int cpl,unsigned int row, unsigned int col, char *ptr)
{
	LCD_address (cpl*row+col);				// set address
   LCD_print(ptr);
}

void LCD_printatAtt (int cpl,unsigned int row, unsigned int col, char *ptr, unsigned char attr)
{
int i;

	LCD_address (cpl*row+col);				// set address
   i=LCD_print(ptr);
	LCD_address (0x1000+cpl*row+col); 	// set attr address
	LCD_WriteCmd(0xb0);						// Auto data write
 	while (i--)
		LCD_WriteAData (attr); 				// write attribute
	LCD_WriteACmd(0xb2);						// reset Auto data write
}

void LCD_CLS()
{
		LCD_address(0); 						// TXT screen
		LCD_fill(30*64,0);					// fill with 0's (clear)
}

/* Plot image data, 8 pixels per byte, MSB left */

void LCD_plot (int x, int y)
{
int strip,bit;
	strip=x>>3;											// strip = x/8
	bit=7-x&'\B0111';									// bit = remainder (MSB is at left)
	y=strip+30*y;
	LCD_address(0x1000+y);							// address strip in gfx screen
	LCD_WriteCmd(0xF8|bit);							// set requested bit (great!)
}

/* Read image files, store them as [(long)lenght][data...] in xmem
Image data for a 240x64 display can be easily extracted from Sun raster
format (without the header). If your display is of the STN- type,
like in this example (many Cika 240x64's are STN-), you'll have to color
invert the picture.
Take a desired image, resize to 240x64, color reduce to 2 colors,
invert colors if STN-. Save to file, remove 32 byte header with your
preferred hex editor.
Otherwise, the BMP trick used for the 320x240 displays still applies,
as the memory format is the same, but I've found the BMP format pads
some extra data at this resolution */

#ximport "rabbit.im2" rabbit

/* To retrieve image data from xmem, we must trade-off speed and buffer size
define a chunk of data long enough (IMGCHUNK), read from xmem, and send it
to the display, repeat for all chunks (IMGCHUNK*CHUNKS=1920 */

#define IMGCHUNK 240
#define CHUNKS	8

void LCD_dump(long imgdata)
{
int x,y;
unsigned char buffer[IMGCHUNK];

		LCD_address(0x1000);						// GFX screen
		LCD_WriteCmd(0xb0);						// Auto data write
		for(y=0;y<CHUNKS;y++){ 					// read data from xmem (skip lenght)
			xmem2root(&buffer,imgdata+IMGCHUNK*y+sizeof(long),IMGCHUNK);
			for(x=0;x<IMGCHUNK;x++)
				LCD_WriteAData(buffer[x]);			// send data to display
      }
		LCD_WriteACmd(0xb2);						// reset Auto data write
}



/* MAIN PROGRAM */

main()
{
int i;



	LCD_init();
	while(1){

		LCD_CLS();		 						// clear TXT screen
		LCD_fillblack();						// black screen
		MsDelay ( 3000 );						// wait 3 secs
		LCD_fillwhite();						// white screen
		MsDelay ( 3000 );						// wait 3 secs

		LCD_printat(30,4,3,"Cika Electronica S.R.L.");  // black text on white background
		MsDelay ( 3000 );						// wait 3 secs
		LCD_fillblack();						// invert image
		MsDelay ( 3000 );						// wait 3 secs
		LCD_CLS();		 						// clear TXT screen

		for(i=0;i<240;i++)
			LCD_plot(i,32); 					// x axis
		for(i=0;i<64;i++)
			LCD_plot(120,i); 					// y axis
		for(i=-120;i<120;i++)
			LCD_plot(120+i,32-(int)(32*(sin(i*3.14159/120)))); // plot sin(x); 0,0 is at top left
		LCD_printat(30,0,0,"sin(x)");
		MsDelay ( 3000 );						// wait 3 secs
		LCD_CLS();		 						// clear TXT screen
		LCD_fillblack();						// black screen
		LCD_WriteCmd(0x84); 					// Attribute mode, internal CG

		LCD_printatAtt(30,1,8,"MODO ATRIBUTOS",0);
		LCD_printatAtt(30,3,8,"Texto Normal",0);
		LCD_printatAtt(30,4,7,"Texto Invertido",5);
		LCD_printatAtt(30,5,3,"Texto Normal Parpadeando",8);
		LCD_printatAtt(30,6,1,"Texto Invertido Parpadeando",13);
		MsDelay ( 6000 );						// wait 6 secs
		LCD_CLS();		 						// clear TXT screen
		LCD_WriteCmd(0x81); 					// TXT XOR GFX mode, internal CG

		LCD_dump(rabbit);
		MsDelay ( 6000 );						// wait 6 secs
	}
}